//+------------------------------------------------------------------+
//|                                    Copyright 2023, Roman Zhitnik |
//|                                           https://tradingkit.net |
//+------------------------------------------------------------------+
#property copyright "Copyright 2023, Roman Zhitnik"
#property link      "https://tradingkit.net"
#property version   "1.00"
//Define the structures that will be used for data from Binance
struct pair_specification_struct
   {
      string symbol;
      int price_precision;
      int volume_precision;
      double contract_size;
      double bid_price;
      double ask_price;
      long time;
   };
struct orders_struct
   {
      long orderId;
      string symbol;
      string side;
      string type;
      double price;
      double size;
      double stopprice;      
   };

struct positions_struct
   {
      string symbol;
      string side;
      double price;
      double size;    
   };

struct balances
   {
      string symbol;
      double amount;
   };
//Import all the functions from the library to gain an ability to use them in your EA or indicator
//Put the BinanceLibrary.ex5 file into the Libraries folder of your MetaTrader 5 terminal
#import "BinanceLibrary.ex5"
//Exchange information
long GetBinanceSpotServerTimestamp(bool binanceUs = false);
long GetBinanceFuturesServerTimestamp();
string GetFuturesExchangeInfo(pair_specification_struct & futures_specs[]);
string GetSpotExchangeInfo(pair_specification_struct & spot_specs[], bool BinanceUS);
string GetCoinmFuturesExchangeInfo(pair_specification_struct & coinm_futures_specs[]);
string GetCoinmFuturesPriceTicker(int offsetHours, pair_specification_struct & coinm_futures_specs[]);
string GetCoinmFuturesPriceTicker(string symbol, int offsetHours, pair_specification_struct & coinm_futures_specs[]);
string GetFuturesPriceTicker(int offsetHours, pair_specification_struct & futures_specs[]);
string GetFuturesPriceTicker(string symbol, int offsetHours, pair_specification_struct & futures_specs[]);
string GetSpotPriceTicker(int offsetHours, pair_specification_struct & spot_specs[], bool BinanceUS);
string GetSpotPriceTicker(string symbol, int offsetHours, pair_specification_struct & spot_specs[], bool BinanceUS);
string CreateOrUpdateSymbol(string symbol, int digitsNumber, string postfix, string marketType, bool binanceUs = false);
//Historical data
string SpotLoadData(string symbol, string postfix, ENUM_TIMEFRAMES period, int daysToLoad, int offsetHours, bool binanceUs = false);
string FuturesLoadData(string symbol, string postfix, ENUM_TIMEFRAMES period, int daysToLoad, int offsetHours);
string CoinmFuturesLoadData(string symbol, string postfix, ENUM_TIMEFRAMES period, int daysToLoad, int offsetHours);
string SpotLoadDataPostexist(string symbol, string postfix, ENUM_TIMEFRAMES period, int daysToLoad, int offsetHours, bool binanceUs = false);
string FuturesLoadDataPostexist(string symbol, string postfix, ENUM_TIMEFRAMES period, int daysToLoad, int offsetHours);
string CoinmFuturesLoadDataPostexist(string symbol, string postfix, ENUM_TIMEFRAMES period, int daysToLoad, int offsetHours);
string SpotLoadLastBar(string symbol, string postfix, ENUM_TIMEFRAMES period, int offsetHours, bool binanceUs = false);
string FuturesLoadLastBar(string symbol, string postfix, ENUM_TIMEFRAMES period, int offsetHours);
string CoinmFuturesLoadLastBar(string symbol, string postfix, ENUM_TIMEFRAMES period, int offsetHours);
//Account
string GetSpotBalances(string apikey, string secretkey, balances & balanceCurrencies[], bool BinanceUS);
string GetFuturesBalances(string apikey, string secretkey, balances & balanceCurrencies[]);
string GetCoinmFuturesBalances(string apikey, string secretkey, balances & balanceCurrencies[]);
string GetFuturesOrders(string apikey, string secretkey, string symbol, orders_struct & ordersArray[]);
string GetCoinmFuturesOrders(string apikey, string secretkey, string symbol, orders_struct & ordersArray[]);
string GetSpotOrders(string apikey, string secretkey, string symbol, orders_struct & ordersArray[], bool BinanceUS);
string GetFuturesPositions(string apikey, string secretkey, string symbol, positions_struct & positionsArray[]);
string GetCoinmFuturesPositions(string apikey, string secretkey, string symbol, positions_struct & positionsArray[]);
string SendSpotOrderToBinance(string symbol = "", string direction = "", string type = "", double lot = 0, double entry_price = 0, double stop_price = 0,string apikey = "", string secretkey = "", bool binanceUs = false);
string SendFuturesOrderToBinance(string symbol = "", string direction = "", string type = "", double lot = 0, double entry_price = 0, double stop_price = 0, double stop_loss = 0, double take_profit = 0, string apikey = "", string secretkey = "");
string SendCoinmFuturesOrderToBinance(string symbol = "", string direction = "", string type = "", double lot = 0, double entry_price = 0, double stop_price = 0, double stop_loss = 0, double take_profit = 0, string apikey = "", string secretkey = "");
string CancelFuturesOrderToBinance(string symbol = "", long orderId = 0, string apikey = "", string secretkey = "");
string CancelCoinmFuturesOrderToBinance(string symbol = "", long orderId = 0, string apikey = "", string secretkey = "");
string CancelSpotOrderToBinance(string symbol = "", long orderId = 0, string apikey = "", string secretkey = "", bool binanceUs = false);
#import
void OnStart()
  {
      string api_key = "your api key";
      string secret_key = "your secret key";
      
      //Declare a struct for further use of exchange instruments information (Futures in current case)
      pair_specification_struct futures[];
      
      //Get price and volume precision information about every single futures instrument
      string exchangeInfoRequest = GetFuturesExchangeInfo(futures);
      if(exchangeInfoRequest!="")Print("Response of Futures Exchange Info request: "+exchangeInfoRequest);
      else Print(futures[0].symbol+" price precision: "+IntegerToString(futures[0].price_precision)+", volume precision: "+IntegerToString(futures[0].volume_precision));
      
      //Get Ask, Bid and Timestamp information about every single futures instrument
      string tickerRequest = GetFuturesPriceTicker(2,futures);      
      if(tickerRequest!="")Print("Response of Futures Price Ticker request: "+tickerRequest);
      else Print(futures[1].symbol+" Bid price: "+DoubleToString(futures[1].bid_price,futures[1].price_precision)+", Ask price: "+DoubleToString(futures[1].ask_price,futures[1].price_precision)+", Timestamp: "+IntegerToString(futures[1].time));       
      
      //Upload all futures from Binance to Symbols list of MetaTrader 5      
      for(int i=0;i<ArraySize(futures);i++)
        {
            CreateOrUpdateSymbol(futures[i].symbol, futures[i].price_precision, ".binf", "Futures", false);
        }
      
      //Download the last 3 days of 1 minute historical data for newly created BTCUSDT.binf instrument with 2 hours offset
      string historicalDataRequest = FuturesLoadData("BTCUSDT", ".binf", PERIOD_M1, 4, 2);
      if(historicalDataRequest!="")Print("Response of BTCUSDT Futures Historical Data request: "+historicalDataRequest);
      
      //Download the missing data from the last available in MetaTrader 5 (limited by daysToLoad parameter) till the current time for the specified instrument
      string absentDataRequest = FuturesLoadDataPostexist("BTCUSDT", ".binf", PERIOD_M1, 1, 2);
      if(absentDataRequest!="")Print("Response of BTCUSDT Futures Absent Historical Data request: "+absentDataRequest);
      
      //Update the OHLCV with the last 1 Minute bar and 2 hours offset for the specified instrument in MetaTrader 5
      string lastBarRequest = FuturesLoadLastBar("BTCUSDT", ".binf", PERIOD_M1, 2);
      if(lastBarRequest!="")Print("Response of BTCUSDT Futures Last Bar Data request: "+lastBarRequest);
      
      balances futures_balances[];
      
      //Fill the declared sctruct with the downloaded balances of Binance futures wallets
      string balancesRequest = GetFuturesBalances(api_key, secret_key, futures_balances);
      for(int i=0;i<ArraySize(futures_balances);i++)
        {
            Print(futures_balances[i].symbol+" balance amount: "+DoubleToString(futures_balances[i].amount));
        }      
      
      //Send a limit order on BNBUSDT futures with 0.02 lots at price 300
      //NOTE!!! IT WILL OPEN A REAL ORDER ON BINANCE EXCHANGE IF THERE IS ENOUGH FUNDS ON YOUR ACCOUNT. REMOVE COMMENT SLASHES AT YOUR OWN RISK
      //string orderSendResult = SendFuturesOrderToBinance("BNBUSDT", "BUY", "LIMIT", 0.05, 200, 0, 0, 0, api_key, secret_key);
      //Print(orderSendResult);
      
      orders_struct orders[];
      //Fill the declared sctruct with the downloaded orders of Binance futures for BNBUSDT pair
      string ordersRequest = GetFuturesOrders(api_key, secret_key, "BNBUSDT", orders);
      for(int i=0;i<ArraySize(orders);i++)
        {
            Print(orders[i].symbol+" type: "+orders[i].type+", orderId: "+IntegerToString(orders[i].orderId)+", price: "+DoubleToString(orders[i].price)+", size: "+DoubleToString(orders[i].size));
        }      
      
      positions_struct positions[];
      //Fill the declared sctruct with the downloaded positions of Binance futures BNBUSDT pair
      string positionsRequest = GetFuturesPositions(api_key, secret_key, "BNBUSDT", positions);
      for(int i=0;i<ArraySize(positions);i++)
        {
            if(positions[i].size!=0)Print(positions[i].symbol+" side: "+positions[i].side+", price: "+DoubleToString(positions[i].price)+", size: "+DoubleToString(positions[i].size));
        }
      
      //Cancel the order on BNBUSDT if it was opened
      if(ArraySize(orders)>0)
         {
            string cancelRequest = CancelFuturesOrderToBinance("BNBUSDT", orders[0].orderId, api_key, secret_key);
            Print(cancelRequest);
         }      
  }